#!/bin/bash

ABS_PATH=$(readlink -e $0)
ABS_PATH=$(dirname $ABS_PATH)

# check if running as root user
if [ $USER != "root" ]; then
    echo "ERROR: This script must be run as root user!"
    exit
fi

# check if online
echo ""
echo -n "Checking if online..."
if [ ! "`ping -q -c 3 google.com 2>/dev/null`" ]; then
    echo "FAIL"
    echo "You need a live internet connection to run this script."
    exit
else
    echo "OK"
fi

# kernel sources: http://highlab.com/~seb/linuxcnc/rtai-for-3.4-prerelease/
# get kernel file names
sleep 3
echo -n "Getting file names from '$ABS_PATH/kernel'..."
headers=""
kernel=""
modules=""
for filepath in "$ABS_PATH/kernel/"*.deb; do
    filename=$(basename $filepath)
    filename=${filename%.*}
    if [[ $filename =~ "linux-headers-" ]]; then
        headers=$filename
    elif [[ $filename =~ "linux-image-" ]]; then
        kernel=$filename
    elif [[ $filename =~ "rtai-modules-" ]]; then
        modules=$filename
    fi
done

# check if kernel files were found OK
if [[ $headers && $kernel && $modules ]]; then
    echo "OK"
    echo "The following .deb files have been found:"
    echo "  $headers"
    echo "  $kernel"
    echo "  $modules"
    echo ""
    # check if kernel is already installed
    resp=$(dpkg --list | grep linux-image)
    for img in $resp; do
        if [[ $img =~ "linux-image" && $kernel =~ $img ]]; then
            echo "WARNING: $kernel is already installed!"
            echo "You may need to reboot to enable it"
            break
        fi
    done
    echo -n "Continue installing this kernel? Y/n: "
    read resp
    if [[ $resp != "Y" && $resp != "y" && $resp != "" ]]; then
        echo "NOT installing kernel!"
        exit
    fi
else
    echo "FAIL"
    echo "Please ensure that '$ABS_PATH/kernel' directory exists AND it contains the following files:"
    echo "  linux-headers-<version>"
    echo "  linux-image-<version>"
    echo "  rtai-modules-<version>"
    echo "You can download these from http://highlab.com/~seb/linuxcnc/rtai-for-3.4-prerelease/"
    exit
fi

# update sources and safe-upgrade O.S.
echo -n "Updating sources..."
resp=$(aptitude update)
substring="Some index files failed to download"
if [[ $resp =~ "$substring" ]]; then
    echo "FAIL"
    echo $resp
    echo "ERROR: the 'aptitude update' command failed:"
    exit
else
    echo "OK"
    echo "Updating system with 'aptitude safe-upgrade'..."
    sleep 3
    aptitude -y safe-upgrade
fi

# install gdebi-core if not already installed
resp=$(gdebi --version &>/dev/null)
if [ $? != 0 ]; then
    echo "Installing gdebi-core..."
    sleep 3
    aptitude -y install gdebi-core
fi

# check gdebi installation
echo -n "Checking if gdebi-core is installed..."
resp=$(gdebi --version &>/dev/null)
if [ $? == 0 ]; then
    echo "OK"
    sleep 3
    echo "Installing headers, kernel and RTAI modules with gdebi..."
    sleep 3
    gdebi --n "$ABS_PATH/kernel/$headers.deb"
    gdebi --n "$ABS_PATH/kernel/$kernel.deb"
    gdebi --n "$ABS_PATH/kernel/$modules.deb"
    # check if kernel installed OK
    echo ""
    echo -n "Checking if $kernel is installed..."
    installed_ok=false
    resp=$(dpkg --list | grep linux-image)
    for img in $resp; do
        if [[ $img =~ "linux-image" && $kernel =~ $img ]]; then
            installed_ok=true
            break
        fi
    done
    if $installed_ok; then
        echo "OK"
        echo "You can now reboot into $kernel"
    else
        echo "FAIL"
        echo "ERROR: Please check the above gdebi output."
    fi
else
    echo "FAIL"
    echo "ERROR: gdebi-core is NOT installed. Try manually installing, then re-run this script."
    exit
fi
exit
